/**
 * @class GenerationDescription
 * @brief Information on generator analyzed
 *
 * -- Class stored in ROOT file --
 *
 * This class contains information regarding the generator used
 * during generation phase. Exactly one copy of this class is
 * constructed for each ROOT file
 */
#ifndef _GenerationDescription_H
#define _GenerationDescription_H

#include <cstring>

#include "TObject.h"
#include "Setup.H"

class GenerationDescription : public TObject {

public:

    int decay_particle; // PDG code of decaying particle

    double bin_min [MAX_DECAY_MULTIPLICITY] [MAX_DECAY_MULTIPLICITY]; // lower bin for i-body decay
    double bin_max [MAX_DECAY_MULTIPLICITY] [MAX_DECAY_MULTIPLICITY]; // upper bin for i-body decay 
    int    nbins   [MAX_DECAY_MULTIPLICITY] [MAX_DECAY_MULTIPLICITY]; // number of bins for i-body decay

    char  gen_desc_1[128]; // description of generator 1 - 1st line
    char  gen_desc_2[128]; // description of generator 1 - 2nd line
    char  gen_desc_3[128]; // description of generator 1 - 3rd line

    char  gen_path[128]; // path of generator
    

    int order_matters; // if set, order of particles in event record matters when searching for decay mode
			       // i.e. [pi- pi0 pi+] differs from [pi- pi+ pi0] . By default: 0
			       

    GenerationDescription();
    GenerationDescription(const Setup &s);
    GenerationDescription(const GenerationDescription &gd);

    ClassDef(GenerationDescription,1) // copy of settings for MC-TESTER generation phase.
};

#endif //_GenerationDescrition_H
