/**
 * @class TDecayMode
 * @brief Contains information regarding decay channel
 *
 * -- Class stored in ROOT file --
 *
 * One copy of this class is created for each decay channel.
 * Includes decay channel name, entries, weights, dispertions, etc.
 */
#ifndef __TDecayMode_H_
#define __TDecayMode_H_
#include "TNamed.h"
#include "THashList.h"
#include "TDirectory.h"
#include "TObjArray.h"
#include "TH1.h"

class HEPParticleList;

class TDecayMode : public TNamed{
    
    private:
	    int mother_id;
	    int num_of_daughters;
	    int daughters[40];
	    long int nentries;
	    double sumw;
	    double sumw2;
	    char latexname[256];
    public:
	    int fill_histos; // if we have less more than 7 stable daughters we won't create histos!
	    TDirectory *my_directory;
    public:	    
	    TObjArray *histograms;
	    //TObjArray *hist2;
	    
	    double fit_parameter;
    public:
		 TDecayMode();
		 TDecayMode(const TDecayMode &x); //shallow copy
	         TDecayMode(int motherid, HEPParticleList *daughterlist);
	virtual ~TDecayMode();

        virtual Int_t Compare(const TObject * other_mode) const;
	virtual int GetMotherId() { return mother_id;};
	virtual int GetNumOfDaughters() { return num_of_daughters;};
	virtual int GetDaughter(int i);	
	virtual int* GetDaughterList() { return daughters;};
	virtual char* GetLaTeXName() { return latexname;};
	virtual TDirectory* GetMyDirectory();
	virtual void SetMyDirectory(TDirectory *d);
	virtual void ls(char *options=0);
	virtual long int GetNEntries(){ return nentries;};
	virtual double GetSumw(){return sumw;};
	virtual double GetSumw2(){return sumw2;};
        virtual void Fill(HEPParticleList *daughters, double weight=1.0);

	virtual void SetNEntries(long int x) { nentries=x; }
	virtual void SetSumw    (long int x) { sumw=x;     }
	virtual void SetSumw2   (long int x) { sumw2=x;    }

  static int NDecayModes;
  static long int NFills;
  static TObjArray *DecayModes;
  static TDecayMode* CheckMode(int motherid, HEPParticleList *daughters, int autoadd=1, int order_matters=0);


  ClassDef(TDecayMode,2) // Decay mode class

};
    
#endif
