#include "TFile.h"
#include "TKey.h"
#include "TSystem.h"
#include "TROOT.h"
#include "TDecayResult.H"
#include <iostream>
using namespace std;
void BOOKLET(TString work_dir);


void BOOKLET()
{
  BOOKLET(".");
}

void BOOKLET(TString work_dir)
{
    //Need to access setup information once again
    if (!gInterpreter->ExecuteMacro(work_dir+"/SETUP.C")){
      cout << "Using default SETUP.C" << endl;
      gInterpreter->ExecuteMacro("SETUP.C");
    }

    // we do not want canvases to be drawn on screen!!!
    gROOT->SetBatch(1);

    int histperline=2;
    int newpage=0;
    gSystem->cd(work_dir);

    gSystem->mkdir("./booklet");

    FILE *texfile=fopen("./booklet.tex","w");

    fprintf(texfile,"%%Generated by MC-TESTER: BOOKLET.C\n");
    fprintf(texfile,"%% to be included in tester.tex\n\n");
	
    TFile *f=TFile::Open("./mc-results.root");
    f->cd();

    TList *directories=f->GetListOfKeys();	
    TIter nextdir(directories);
    TKey *key_dir=0;

    char histname[128];
    char dirname[128];
    int histcnt=0;
    
    // loop over directories
    while (key_dir=(TKey*) nextdir() ) { 
	    
	TDirectory *subdir=(TDirectory*) f->Get(key_dir->GetName());

	printf("We are in %s\n",subdir->GetName());
	sprintf(dirname,"./booklet/%s",subdir->GetName());
	

	//now replace all _ to @
	for (int i=0;i<strlen(dirname);i++) if (dirname[i]=='_') dirname[i]='@';
	//all ~ to !
	for (int i=0;i<strlen(dirname);i++) if (dirname[i]=='~') dirname[i]='.';
	//and spaces to ,
	for (int i=0;i<strlen(dirname);i++) if (dirname[i]==' ') dirname[i]='.';
	// and "=>" to  "TO"
	for (int i=0;i<strlen(dirname)-1;i++) {
	    if ( dirname[i] == '=' && dirname[i+1] == '>' ) {
	    
		dirname[i]='T';
		dirname[i+1]='O';
	    }
	}
	gSystem->mkdir(dirname);

	if (strcmp(subdir->GetTitle(),"USER_HISTOGRAMS")==0) {
	    if (subdir->GetListOfKeys() && subdir->GetListOfKeys()->GetSize() > 0) {
	      TString mode_label = subdir->GetName();
	      fprintf(texfile,"\\section{User Histograms}\n \\label{%s}",(const char *)mode_label.ReplaceAll("~","tilda"));
	    } else {
		continue;
	    }
	} else if(subdir->GetListOfKeys() && subdir->GetListOfKeys()->GetSize() > 1){
	    fprintf(texfile,"\\section{Decay Channel: $%s$}\n",subdir->GetTitle());
	    TString mode_label = subdir->GetName();
	    fprintf(texfile,"\\label{%s}",(const char *)mode_label.ReplaceAll("~","tilda"));
	}

	TList   *objects=subdir->GetListOfKeys();

	// now we want to find TDecayMode object...
	TIter    nextobj(objects);
	TKey    *key_obj=0;
	TDecayResult *dr=0;

	int description_printed=0;
	
	// within subdirectory, loop over all objects:
	while (key_obj=(TKey*)nextobj()) {
	    if (strcmp(key_obj->GetClassName(),"TDecayResult")==0 ){
		dr=(TDecayResult*) key_obj->ReadObj();
		if(Setup::use_log_y) dr->Draw("LOGY");
		else dr->Draw();
		dr->c->SetFillColor(10);
		histcnt++;
		sprintf(histname,"%s/%s.eps",dirname,dr->GetName());
		////now replace all _ to @
		for (int i=0;i<strlen(histname);i++) if (histname[i]=='_') histname[i]='@';

		dr->c->SaveAs(histname);


	
		if (!description_printed) {
		    long n1 = (long)dr->h1->GetEntries();
		    long n2 = (long)dr->h2->GetEntries();
		    description_printed=1;
		    fprintf(texfile,"Number of events from \\textcolor{red}{generator 1}: %li\n ",n1);
		    if( Setup::scale_histograms && n1<n2 ) fprintf(texfile,"{\\it (scaled to generator2)}\n ");
		    fprintf(texfile,"\\newline\n");
		    fprintf(texfile,"Number of events from \\textcolor{green}{generator 2}: %li\n ",n2);
		    if( Setup::scale_histograms && n2<n1 ) fprintf(texfile,"{\\it (scaled to generator1)}\n ");
		    fprintf(texfile,"\\newline\n");
		    fprintf(texfile,"\\newline\n");

		}

		fprintf(texfile,"{ \\resizebox*{%4.2f\\textwidth}{!}{\\includegraphics{%s}} }\n",(1.0/histperline)-0.01,histname);


	    }
	}

	if (newpage) fprintf(texfile,"\\newpage\n");
    
    }; // end of loop over directories
	
fclose(texfile);

}
